from pyspedas.utilities.dailynames import dailynames
from pyspedas.utilities.download import download
from pytplot import time_clip as tclip
from pytplot import cdf_to_tplot

from .config import CONFIG

def load(trange=['2018-11-5', '2018-11-6'], 
         probe=['noaa19'],
         instrument='sem', 
         datatype='*', 
         prefix='',
         suffix='',
         get_support_data=False, 
         varformat=None,
         varnames=[],
         downloadonly=False,
         notplot=False,
         no_update=False,
         time_clip=False,
         force_download=False):
    """
    This function loads POES Space Environment Monitor data. This function is
    not meant to be called directly; instead, see the wrapper:
        pyspedas.poes.sem

    Parameters
    ----------
        trange : list of str, default=['2018-11-5', '2018-11-6']
            time range of interest [starttime, endtime] with the format
            'YYYY-MM-DD','YYYY-MM-DD'] or to specify more or less than a day
            ['YYYY-MM-DD/hh:mm:ss','YYYY-MM-DD/hh:mm:ss']

        probe: str or list of str, default=['noaa19']
            POES spacecraft name(s); e.g., metop1, metop2, noaa15, noaa16,
            noaa18, noaa19

        instrument: str, default='sem'
             Name of the instrument.

        datatype: str, default='*'
            This variable is unused. It is reserved for the future use.

        prefix: str
            The tplot variable names will be given this prefix.  By default, no prefix is added.
            Default: ''

        suffix: str, optional
            The tplot variable names will be given this suffix.  By default,
            no suffix is added.

        get_support_data: bool, default=False
            Data with an attribute "VAR_TYPE" with a value of "support_data"
            will be loaded into tplot.  By default, only loads in data with a
            "VAR_TYPE" attribute of "data".

        varformat: str, default=False
            The file variable formats to load into tplot.  Wildcard character
            "*" is accepted.  By default, all variables are loaded in.

        varnames: list of str, optional
            List of variable names to load (if not specified,
            all data variables are loaded)

        downloadonly: bool, default=False
            Set this flag to download the CDF files, but not load them into
            tplot variables

        notplot: bool, default=False
            Return the data in hash tables instead of creating tplot variables

        no_update: bool, default=False
            If set, only load data from your local cache

        time_clip: bool, default=False
            Time clip the variables to exactly the range specified in the trange keyword

        force_download: bool
            Download file even if local version is more recent than server version
            Default: False

    Returns
    -------
    dict or list
        List of tplot variables created.

    Examples
    --------
    This function is not intended to be called directly.
    """

    if not isinstance(probe, list):
        probe = [probe]

    out_files = []

    for prb in probe:
        if instrument == 'sem':
            pathformat = prb + '/sem2_fluxes-2sec/%Y/' + prb + '_poes-sem2_fluxes-2sec_%Y%m%d_v??.cdf'

        # find the full remote path names using the trange
        remote_names = dailynames(file_format=pathformat, trange=trange)

        files = download(remote_file=remote_names, remote_path=CONFIG['remote_data_dir'], local_path=CONFIG['local_data_dir'], no_download=no_update, force_download=force_download)
        if files is not None:
            for file in files:
                out_files.append(file)

    out_files = sorted(out_files)

    if downloadonly:
        return out_files

    tvars = cdf_to_tplot(out_files, prefix=prefix, suffix=suffix, get_support_data=get_support_data, varformat=varformat, varnames=varnames, notplot=notplot)

    if notplot:
        return tvars

    if time_clip:
        for new_var in tvars:
            tclip(new_var, trange[0], trange[1], suffix='')

    return tvars



from .load import load

# This routine was originally in poes/__init__.py.
def sem(trange=['2018-11-5', '2018-11-6'],
        probe=['noaa19'],
        datatype=None,
        prefix='',
        suffix='',
        get_support_data=False,
        varformat=None,
        varnames=[],
        downloadonly=False,
        notplot=False,
        no_update=False,
        time_clip=False,
        force_download=False):
    """
    This function loads POES Space Environment Monitor (SEM) data

    Parameters
    ----------
        trange : list of str
            time range of interest [starttime, endtime] with the format
            ['YYYY-MM-DD','YYYY-MM-DD'] or to specify more or less than a day
            ['YYYY-MM-DD/hh:mm:ss','YYYY-MM-DD/hh:mm:ss']
            Default: ['2018-11-5', '2018-11-6']

        probe: str or list of str
            POES spacecraft name(s); e.g., metop1, metop2, noaa15, noaa16,
            noaa18, noaa19. Default: noaa19

        datatype: str, optional
            This variable is unused. It is reserved for the future use.

        prefix: str
            The tplot variable names will be given this prefix.  By default, no prefix is added.
            Default: ''

        suffix: str, optional
            The tplot variable names will be given this suffix.  By default,
            no suffix is added.

        get_support_data: bool, default=False
            If True, data with an attribute "VAR_TYPE" with a value of "support_data"
            will be loaded into tplot. Default: False

        varformat: str
            If specified,  file variable formats to load into tplot.  Wildcard characters
            `*` and `?` are accepted.  By default, all variables are loaded in.

        varnames: list of str, optional
            List of variable names to load (if not specified,
            all data variables are loaded)

        downloadonly: bool
            Set this flag to download the CDF files, but not load them into
            tplot variables. Default: False

        notplot: bool
            Return the data in hash tables instead of creating tplot variables. Default: False

        no_update: bool
            If set, only load data from your local cache. Default: False

        time_clip: bool
            Time clip the variables to exactly the range specified in the trange keyword. Default: False

        force_download: bool
            Download file even if local version is more recent than server version
            Default: False

    Returns
    -------
    dict or list
        List of tplot variables created.

    Examples
    --------
    >>> sem_vars = pyspedas.poes.sem(trange=['2013-11-5', '2013-11-6'])
    >>> tplot('ted_ele_tel30_low_eflux')
    """
    return load(instrument='sem', probe=probe, trange=trange, datatype=datatype, prefix=prefix, suffix=suffix, get_support_data=get_support_data, varformat=varformat, varnames=varnames, downloadonly=downloadonly, notplot=notplot, time_clip=time_clip, no_update=no_update, force_download=force_download)



from functools import update_wrapper
from pyspedas.utilities.pyspedas_functools import better_partial
from .load import load
from pyspedas.utilities.datasets import find_datasets


# Define partial wrappers for other load routines, fixing the instrument parameter
# 'better_partial' works better with PyCharm autocompletion than functools.partial
# update_wrapper() is necessary for help() to show info for the wrapped function,
# rather than the partial() object.

dcf = better_partial(load, instrument="dcf")
update_wrapper(dcf, load)
acf = better_partial(load, instrument="acf")
update_wrapper(acf, load)
esa = better_partial(load, instrument="esa")
update_wrapper(esa, load)
teams = better_partial(load, instrument="teams")
update_wrapper(teams, load)

datasets = better_partial(find_datasets, mission="FAST", label=True)
update_wrapper(datasets, find_datasets)



"""A setuptools based setup module."""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages

# To use a consistent encols distding
from codecs import open

setup(
    name="pyspedas",
    version="1.6.0",
    description="Python Space Physics Environment Data Analysis Software (pySPEDAS)",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/spedas/pyspedas",
    author="Jim Lewis",
    author_email="jwl@ssl.berkeley.edu",
    license="MIT",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
    ],
    keywords="spedas data tools",
    project_urls={
        "Information": "http://spedas.org/wiki/",
    },
    packages=find_packages(exclude=["contrib", "docs", "tests*"]),
    install_requires=[
        "numpy>=1.20.0",
        "requests",
        "geopack>=1.0.11",
        "scipy",
        "cdflib",
        "cdasws>=1.7.24",
        "netCDF4>=1.6.2",
        "pywavelets",
        "astropy",
        "hapiclient>=0.2.2",
        "pytplot-mpl-temp>=2.2.41",
        "viresclient",
    ],
    python_requires=">=3.9",
    include_package_data=True,
)
